@@ -107,12 +107,12 @@ static inline void CopyAndConvertOut(void *dst, jack_sample_t *src, size_t nfram
             s32dst += channel;
             sample_move_d24_sS((char*)s32dst, src, nframes, chcount<<2, NULL); // No dithering for now...
 			break;
-        }    
+        }
 		case 32: {
             signed int *s32dst = (signed int*)dst;
             s32dst += channel;
             sample_move_d32u24_sS((char*)s32dst, src, nframes, chcount<<2, NULL);
-			break;  
+			break;
         }
 	}
 }
@@ -126,7 +126,7 @@ void JackOSSDriver::SetSampleFormat()
             fSampleSize = sizeof(int);
 			break;
 		case 32:	/* native-endian 32-bit integer */
-            fSampleFormat = AFMT_S32_NE; 
+            fSampleFormat = AFMT_S32_NE;
             fSampleSize = sizeof(int);
 			break;
 		case 16:	/* native-endian 16-bit integer */
@@ -143,13 +143,13 @@ void JackOSSDriver::DisplayDeviceInfo()
     oss_audioinfo ai_in, ai_out;
     memset(&info, 0, sizeof(audio_buf_info));
     int cap = 0;
-    
+
     // Duplex cards : http://manuals.opensound.com/developer/full_duplex.html
     jack_info("Audio Interface Description :");
     jack_info("Sampling Frequency : %d, Sample Format : %d, Mode : %d", fEngineControl->fSampleRate, fSampleFormat, fRWMode);
-    
+
     if (fRWMode & kWrite) {
-	
+
         oss_sysinfo si;
         if (ioctl(fOutFD, OSS_SYSINFO, &si) == -1) {
             jack_error("JackOSSDriver::DisplayDeviceInfo OSS_SYSINFO failed : %s@%i, errno = %d", __FILE__, __LINE__, errno);
@@ -161,17 +161,17 @@ void JackOSSDriver::DisplayDeviceInfo()
             jack_info("OSS numaudioengines %d", si.numaudioengines);
             jack_info("OSS numcards %d", si.numcards);
         }
-	
+
         jack_info("Output capabilities - %d channels : ", fPlaybackChannels);
         jack_info("Output block size = %d", fOutputBufferSize);
-    
+
         if (ioctl(fOutFD, SNDCTL_DSP_GETOSPACE, &info) == -1)  {
             jack_error("JackOSSDriver::DisplayDeviceInfo SNDCTL_DSP_GETOSPACE failed : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         } else {
-            jack_info("output space info: fragments = %d, fragstotal = %d, fragsize = %d, bytes = %d", 
+            jack_info("output space info: fragments = %d, fragstotal = %d, fragsize = %d, bytes = %d",
                 info.fragments, info.fragstotal, info.fragsize, info.bytes);
         }
-    
+
         if (ioctl(fOutFD, SNDCTL_DSP_GETCAPS, &cap) == -1)  {
             jack_error("JackOSSDriver::DisplayDeviceInfo SNDCTL_DSP_GETCAPS failed : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         } else {
@@ -184,10 +184,10 @@ void JackOSSDriver::DisplayDeviceInfo()
             if (cap & DSP_CAP_MULTI) 	jack_info(" DSP_CAP_MULTI");
             if (cap & DSP_CAP_BIND) 	jack_info(" DSP_CAP_BIND");
         }
-    }	
-    
+    }
+
     if (fRWMode & kRead) {
-    
+
       	oss_sysinfo si;
         if (ioctl(fInFD, OSS_SYSINFO, &si) == -1) {
             jack_error("JackOSSDriver::DisplayDeviceInfo OSS_SYSINFO failed : %s@%i, errno = %d", __FILE__, __LINE__, errno);
@@ -199,14 +199,14 @@ void JackOSSDriver::DisplayDeviceInfo()
             jack_info("OSS numaudioengines %d", si.numaudioengines);
             jack_info("OSS numcards %d", si.numcards);
         }
-    
+
         jack_info("Input capabilities - %d channels : ", fCaptureChannels);
         jack_info("Input block size = %d", fInputBufferSize);
-    
-        if (ioctl(fInFD, SNDCTL_DSP_GETISPACE, &info) == -1) { 
+
+        if (ioctl(fInFD, SNDCTL_DSP_GETISPACE, &info) == -1) {
             jack_error("JackOSSDriver::DisplayDeviceInfo SNDCTL_DSP_GETOSPACE failed : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         } else {
-            jack_info("input space info: fragments = %d, fragstotal = %d, fragsize = %d, bytes = %d", 
+            jack_info("input space info: fragments = %d, fragstotal = %d, fragsize = %d, bytes = %d",
                 info.fragments, info.fragstotal, info.fragsize, info.bytes);
         }
 
@@ -223,7 +223,7 @@ void JackOSSDriver::DisplayDeviceInfo()
             if (cap & DSP_CAP_BIND) 	jack_info(" DSP_CAP_BIND");
         }
     }
-     
+
     if (ai_in.rate_source != ai_out.rate_source) {
         jack_info("Warning : input and output are not necessarily driven by the same clock!");
     }
@@ -238,7 +238,7 @@ int JackOSSDriver::OpenInput()
     jack_nframes_t cur_sample_rate;
 
     if (fCaptureChannels == 0) fCaptureChannels = 2;
-  
+
     if ((fInFD = open(fCaptureDriverName, O_RDONLY | ((fExcl) ? O_EXCL : 0))) < 0) {
         jack_error("JackOSSDriver::OpenInput failed to open device : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         return -1;
@@ -253,7 +253,7 @@ int JackOSSDriver::OpenInput()
         }
     }
 
-    gFragFormat = (2 << 16) + int2pow2(fEngineControl->fBufferSize * fSampleSize * fCaptureChannels);	
+    gFragFormat = (2 << 16) + int2pow2(fEngineControl->fBufferSize * fSampleSize * fCaptureChannels);
     if (ioctl(fInFD, SNDCTL_DSP_SETFRAGMENT, &gFragFormat) == -1) {
         jack_error("JackOSSDriver::OpenInput failed to set fragments : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         goto error;
@@ -267,7 +267,7 @@ int JackOSSDriver::OpenInput()
     if (cur_sample_format != fSampleFormat) {
         jack_info("JackOSSDriver::OpenInput driver forced the sample format %ld", fSampleFormat);
     }
-    
+
     cur_capture_channels = fCaptureChannels;
     if (ioctl(fInFD, SNDCTL_DSP_CHANNELS, &fCaptureChannels) == -1) {
         jack_error("JackOSSDriver::OpenInput failed to set channels : %s@%i, errno = %d", __FILE__, __LINE__, errno);
@@ -276,7 +276,7 @@ int JackOSSDriver::OpenInput()
     if (cur_capture_channels != fCaptureChannels) {
         jack_info("JackOSSDriver::OpenInput driver forced the number of capture channels %ld", fCaptureChannels);
     }
-       
+
     cur_sample_rate = fEngineControl->fSampleRate;
     if (ioctl(fInFD, SNDCTL_DSP_SPEED, &fEngineControl->fSampleRate) == -1) {
         jack_error("JackOSSDriver::OpenInput failed to set sample rate : %s@%i, errno = %d", __FILE__, __LINE__, errno);
@@ -291,7 +291,7 @@ int JackOSSDriver::OpenInput()
         jack_error("JackOSSDriver::OpenInput failed to get fragments : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         goto error;
     }
-        
+
     if (fInputBufferSize != fEngineControl->fBufferSize * fSampleSize * fCaptureChannels) {
        if (fIgnoreHW) {
            int new_buffer_size = fInputBufferSize / (fSampleSize * fCaptureChannels);
@@ -306,7 +306,7 @@ int JackOSSDriver::OpenInput()
     fInputBuffer = (void*)calloc(fInputBufferSize, 1);
     assert(fInputBuffer);
     return 0;
-    
+
 error:
     ::close(fInFD);
     return -1;
@@ -321,27 +321,27 @@ int JackOSSDriver::OpenOutput()
     jack_nframes_t cur_sample_rate;
 
     if (fPlaybackChannels == 0) fPlaybackChannels = 2;
-    
+
     if ((fOutFD = open(fPlaybackDriverName, O_WRONLY | ((fExcl) ? O_EXCL : 0))) < 0) {
        jack_error("JackOSSDriver::OpenOutput failed to open device : %s@%i, errno = %d", __FILE__, __LINE__, errno);
        return -1;
     }
 
     jack_log("JackOSSDriver::OpenOutput output fOutFD = %d", fOutFD);
-    
+
     if (fExcl) {
         if (ioctl(fOutFD, SNDCTL_DSP_COOKEDMODE, &flags) == -1) {
             jack_error("JackOSSDriver::OpenOutput failed to set cooked mode : %s@%i, errno = %d", __FILE__, __LINE__, errno);
             goto error;
-        }  
-    } 
+        }
+    }
 
-    gFragFormat = (2 << 16) + int2pow2(fEngineControl->fBufferSize * fSampleSize * fPlaybackChannels);	
+    gFragFormat = (2 << 16) + int2pow2(fEngineControl->fBufferSize * fSampleSize * fPlaybackChannels);
     if (ioctl(fOutFD, SNDCTL_DSP_SETFRAGMENT, &gFragFormat) == -1) {
         jack_error("JackOSSDriver::OpenOutput failed to set fragments : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         goto error;
     }
-   
+
     cur_sample_format = fSampleFormat;
     if (ioctl(fOutFD, SNDCTL_DSP_SETFMT, &fSampleFormat) == -1) {
         jack_error("JackOSSDriver::OpenOutput failed to set format : %s@%i, errno = %d", __FILE__, __LINE__, errno);
@@ -350,7 +350,7 @@ int JackOSSDriver::OpenOutput()
     if (cur_sample_format != fSampleFormat) {
         jack_info("JackOSSDriver::OpenOutput driver forced the sample format %ld", fSampleFormat);
     }
-    
+
     cur_playback_channels = fPlaybackChannels;
     if (ioctl(fOutFD, SNDCTL_DSP_CHANNELS, &fPlaybackChannels) == -1) {
         jack_error("JackOSSDriver::OpenOutput failed to set channels : %s@%i, errno = %d", __FILE__, __LINE__, errno);
@@ -374,7 +374,7 @@ int JackOSSDriver::OpenOutput()
         jack_error("JackOSSDriver::OpenOutput failed to get fragments : %s@%i, errno = %d", __FILE__, __LINE__, errno);
         goto error;
     }
-         
+
     if (fOutputBufferSize != fEngineControl->fBufferSize * fSampleSize * fPlaybackChannels) {
        if (fIgnoreHW) {
            int new_buffer_size = fOutputBufferSize / (fSampleSize * fPlaybackChannels);
@@ -385,19 +385,19 @@ int JackOSSDriver::OpenOutput()
            goto error;
        }
     }
-    
+
     fOutputBuffer = (void*)calloc(fOutputBufferSize, 1);
     fFirstCycle = true;
     assert(fOutputBuffer);
     return 0;
-    
+
 error:
     ::close(fOutFD);
     return -1;
 }
 
 int JackOSSDriver::Open(jack_nframes_t nframes,
-                      int user_nperiods, 
+                      int user_nperiods,
                       jack_nframes_t samplerate,
                       bool capturing,
                       bool playing,
@@ -413,7 +413,7 @@ int JackOSSDriver::Open(jack_nframes_t nframes,
                       bool ignorehwbuf)
 {
     // Generic JackAudioDriver Open
-    if (JackAudioDriver::Open(nframes, samplerate, capturing, playing, inchannels, outchannels, monitor, 
+    if (JackAudioDriver::Open(nframes, samplerate, capturing, playing, inchannels, outchannels, monitor,
         capture_driver_uid, playback_driver_uid, capture_latency, playback_latency) != 0) {
         return -1;
     } else {
@@ -422,7 +422,7 @@ int JackOSSDriver::Open(jack_nframes_t nframes,
             jack_error("Cannot run in asynchronous mode, use the -S parameter for jackd");
             return -1;
         }
-     
+
         fRWMode |= ((capturing) ? kRead : 0);
         fRWMode |= ((playing) ? kWrite : 0);
         fBits = bits;
@@ -434,7 +434,7 @@ int JackOSSDriver::Open(jack_nframes_t nframes,
         // Force memory page in
         memset(&gCycleTable, 0, sizeof(gCycleTable));
     #endif
-        
+
         if (OpenAux() < 0) {
             Close();
             return -1;
@@ -448,14 +448,14 @@ int JackOSSDriver::Close()
 {
  #ifdef JACK_MONITOR
     FILE* file = fopen("OSSProfiling.log", "w");
-    
+
     if (file) {
         jack_info("Writing OSS driver timing data....");
         for (int i = 1; i < gCycleCount; i++) {
             int d1 = gCycleTable.fTable[i].fAfterRead - gCycleTable.fTable[i].fBeforeRead;
             int d2 = gCycleTable.fTable[i].fAfterReadConvert - gCycleTable.fTable[i].fAfterRead;
             int d3 = gCycleTable.fTable[i].fAfterWrite - gCycleTable.fTable[i].fBeforeWrite;
-            int d4 = gCycleTable.fTable[i].fBeforeWrite - gCycleTable.fTable[i].fBeforeWriteConvert;     
+            int d4 = gCycleTable.fTable[i].fBeforeWrite - gCycleTable.fTable[i].fBeforeWriteConvert;
             fprintf(file, "%d \t %d \t %d \t %d \t \n", d1, d2, d3, d4);
         }
         fclose(file);
@@ -468,7 +468,7 @@ int JackOSSDriver::Close()
     if (file == NULL) {
         jack_error("JackOSSDriver::Close cannot open TimingOSS.plot file");
     } else {
-        
+
         fprintf(file, "set grid\n");
         fprintf(file, "set title \"OSS audio driver timing\"\n");
         fprintf(file, "set xlabel \"audio cycles\"\n");
@@ -477,10 +477,10 @@ int JackOSSDriver::Close()
                             \"OSSProfiling.log\" using 2 title \"Driver read convert duration\" with lines, \
                             \"OSSProfiling.log\" using 3 title \"Driver write wait\" with lines, \
                             \"OSSProfiling.log\" using 4 title \"Driver write convert duration\" with lines\n");
-    
+
         fprintf(file, "set output 'TimingOSS.pdf\n");
         fprintf(file, "set terminal pdf\n");
-    
+
         fprintf(file, "set grid\n");
         fprintf(file, "set title \"OSS audio driver timing\"\n");
         fprintf(file, "set xlabel \"audio cycles\"\n");
@@ -489,12 +489,12 @@ int JackOSSDriver::Close()
                             \"OSSProfiling.log\" using 2 title \"Driver read convert duration\" with lines, \
                             \"OSSProfiling.log\" using 3 title \"Driver write wait\" with lines, \
                             \"OSSProfiling.log\" using 4 title \"Driver write convert duration\" with lines\n");
-      
+
         fclose(file);
     }
  #endif
     int res = JackAudioDriver::Close();
-    CloseAux();    
+    CloseAux();
     return res;
 }
 
@@ -506,8 +506,8 @@ int JackOSSDriver::OpenAux()
     if ((fRWMode & kRead) && (OpenInput() < 0)) {
         return -1;
     }
-  
-    if ((fRWMode & kWrite) && (OpenOutput() < 0)) { 
+
+    if ((fRWMode & kWrite) && (OpenOutput() < 0)) {
         return -1;
     }
 
@@ -522,7 +522,7 @@ int JackOSSDriver::OpenAux()
     }
     */
 
-    DisplayDeviceInfo();  
+    DisplayDeviceInfo();
     return 0;
 }
 
@@ -532,16 +532,16 @@ void JackOSSDriver::CloseAux()
         close(fInFD);
         fInFD = -1;
     }
-    
+
     if (fRWMode & kWrite && fOutFD > 0) {
         close(fOutFD);
         fOutFD = -1;
     }
-    
+
     if (fInputBuffer)
         free(fInputBuffer);
     fInputBuffer = NULL;
-    
+
     if (fOutputBuffer)
         free(fOutputBuffer);
     fOutputBuffer = NULL;
@@ -554,7 +554,7 @@ int JackOSSDriver::Read()
         JackDriver::CycleTakeBeginTime();
         return 0;
     }
-    
+
     ssize_t count;
 
 #ifdef JACK_MONITOR
@@ -562,28 +562,28 @@ int JackOSSDriver::Read()
 #endif
 
     audio_errinfo ei_in;
-    count = ::read(fInFD, fInputBuffer, fInputBufferSize);  
- 
+    count = ::read(fInFD, fInputBuffer, fInputBufferSize);
+
 #ifdef JACK_MONITOR
     if (count > 0 && count != (int)fInputBufferSize)
         jack_log("JackOSSDriver::Read count = %ld", count / (fSampleSize * fCaptureChannels));
     gCycleTable.fTable[gCycleCount].fAfterRead = GetMicroSeconds();
 #endif
-    
+
     // XRun detection
     if (ioctl(fInFD, SNDCTL_DSP_GETERROR, &ei_in) == 0) {
 
         if (ei_in.rec_overruns > 0 ) {
             jack_error("JackOSSDriver::Read overruns");
             jack_time_t cur_time = GetMicroSeconds();
-            NotifyXRun(cur_time, float(cur_time - fBeginDateUst));   // Better this value than nothing... 
+            NotifyXRun(cur_time, float(cur_time - fBeginDateUst));   // Better this value than nothing...
         }
 
         if (ei_in.rec_errorcount > 0 && ei_in.rec_lasterror != 0) {
             jack_error("%d OSS rec event(s), last=%05d:%d", ei_in.rec_errorcount, ei_in.rec_lasterror, ei_in.rec_errorparm);
         }
-    }   
-    
+    }
+
     if (count < 0) {
         jack_log("JackOSSDriver::Read error = %s", strerror(errno));
         return -1;
@@ -603,7 +603,7 @@ int JackOSSDriver::Read()
     #ifdef JACK_MONITOR
         gCycleTable.fTable[gCycleCount].fAfterReadConvert = GetMicroSeconds();
     #endif
-        
+
         return 0;
     }
 }
@@ -618,10 +618,10 @@ int JackOSSDriver::Write()
 
     ssize_t count;
     audio_errinfo ei_out;
-   
-    // Maybe necessary to write an empty output buffer first time : see http://manuals.opensound.com/developer/fulldup.c.html   
+
+    // Maybe necessary to write an empty output buffer first time : see http://manuals.opensound.com/developer/fulldup.c.html
     if (fFirstCycle) {
-    
+
         fFirstCycle = false;
         memset(fOutputBuffer, 0, fOutputBufferSize);
 
@@ -633,17 +633,17 @@ int JackOSSDriver::Write()
                 return -1;
             }
         }
-        
+
         int delay;
         if (ioctl(fOutFD, SNDCTL_DSP_GETODELAY, &delay) == -1) {
             jack_error("JackOSSDriver::Write error get out delay : %s@%i, errno = %d", __FILE__, __LINE__, errno);
             return -1;
         }
-        
+
         delay /= fSampleSize * fPlaybackChannels;
         jack_info("JackOSSDriver::Write output latency frames = %ld", delay);
     }
-  
+
 #ifdef JACK_MONITOR
     gCycleTable.fTable[gCycleCount].fBeforeWriteConvert = GetMicroSeconds();
 #endif
@@ -657,7 +657,7 @@ int JackOSSDriver::Write()
 
   #ifdef JACK_MONITOR
     gCycleTable.fTable[gCycleCount].fBeforeWrite = GetMicroSeconds();
-  #endif    
+  #endif
 
     // Keep end cycle time
     JackDriver::CycleTakeEndTime();
@@ -676,14 +676,14 @@ int JackOSSDriver::Write()
         if (ei_out.play_underruns > 0) {
             jack_error("JackOSSDriver::Write underruns");
             jack_time_t cur_time = GetMicroSeconds();
-            NotifyXRun(cur_time, float(cur_time - fBeginDateUst));   // Better this value than nothing... 
+            NotifyXRun(cur_time, float(cur_time - fBeginDateUst));   // Better this value than nothing...
         }
 
         if (ei_out.play_errorcount > 0 && ei_out.play_lasterror != 0) {
             jack_error("%d OSS play event(s), last=%05d:%d",ei_out.play_errorcount, ei_out.play_lasterror, ei_out.play_errorparm);
         }
     }
-     
+
     if (count < 0) {
         jack_log("JackOSSDriver::Write error = %s", strerror(errno));
         return -1;
@@ -697,15 +697,15 @@ int JackOSSDriver::Write()
 
 int JackOSSDriver::SetBufferSize(jack_nframes_t buffer_size)
 {
-    CloseAux(); 
-    JackAudioDriver::SetBufferSize(buffer_size); // never fails
+    CloseAux();
+    JackAudioDriver::SetBufferSize(buffer_size); // Generic change, never fails
     return OpenAux();
 }
 
 int JackOSSDriver::ProcessSync()
 {
     // Read input buffers for the current cycle
-    if (Read() < 0) { 
+    if (Read() < 0) {
         jack_error("ProcessSync: read error, skip cycle");
         return 0;   // Non fatal error here, skip cycle, but continue processing...
     }
@@ -713,15 +713,15 @@ int JackOSSDriver::ProcessSync()
     if (fIsMaster) {
         ProcessGraphSync();
     } else {
-        fGraphManager->ResumeRefNum(&fClientControl, fSynchroTable);
+        ResumeRefNum();
     }
-    
+
     // Write output buffers for the current cycle
-    if (Write() < 0) { 
+    if (Write() < 0) {
         jack_error("JackAudioDriver::ProcessSync: write error, skip cycle");
         return 0;   // Non fatal error here, skip cycle, but continue processing...
     }
-    
+
     return 0;
 }
 
@@ -734,119 +734,44 @@ extern "C"
 
 SERVER_EXPORT jack_driver_desc_t* driver_get_descriptor()
 {
-    jack_driver_desc_t *desc;
-    unsigned int i;
-    desc = (jack_driver_desc_t*)calloc(1, sizeof(jack_driver_desc_t));
+    jack_driver_desc_t * desc;
+    jack_driver_desc_filler_t filler;
+    jack_driver_param_value_t value;
 
-    strcpy(desc->name, "oss");                              // size MUST be less then JACK_DRIVER_NAME_MAX + 1
-    strcpy(desc->desc, "OSS API based audio backend");      // size MUST be less then JACK_DRIVER_PARAM_DESC + 1
-       
-    desc->nparams = OSS_DRIVER_N_PARAMS;
-    desc->params = (jack_driver_param_desc_t*)calloc(desc->nparams, sizeof(jack_driver_param_desc_t));
+    desc = jack_driver_descriptor_construct("oss", JackDriverMaster, "OSS API based audio backend", &filler);
 
-    i = 0;
-    strcpy(desc->params[i].name, "rate");
-    desc->params[i].character = 'r';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.ui = OSS_DRIVER_DEF_FS;
-    strcpy(desc->params[i].short_desc, "Sample rate");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.ui = OSS_DRIVER_DEF_FS;
+    jack_driver_descriptor_add_parameter(desc, &filler, "rate", 'r', JackDriverParamUInt, &value, NULL, "Sample rate", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "period");
-    desc->params[i].character = 'p';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.ui = OSS_DRIVER_DEF_BLKSIZE;
-    strcpy(desc->params[i].short_desc, "Frames per period");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.ui = OSS_DRIVER_DEF_BLKSIZE;
+    jack_driver_descriptor_add_parameter(desc, &filler, "period", 'p', JackDriverParamUInt, &value, NULL, "Frames per period", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "nperiods");
-    desc->params[i].character = 'n';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.ui = OSS_DRIVER_DEF_NPERIODS;
-    strcpy(desc->params[i].short_desc, "Number of periods to prefill output buffer");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.ui = OSS_DRIVER_DEF_NPERIODS;
+    jack_driver_descriptor_add_parameter(desc, &filler, "nperiods", 'n', JackDriverParamUInt, &value, NULL, "Number of periods to prefill output buffer", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "wordlength");
-    desc->params[i].character = 'w';
-    desc->params[i].type = JackDriverParamInt;
-    desc->params[i].value.i = OSS_DRIVER_DEF_BITS;
-    strcpy(desc->params[i].short_desc, "Word length");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.i = OSS_DRIVER_DEF_BITS;
+    jack_driver_descriptor_add_parameter(desc, &filler, "wordlength", 'w', JackDriverParamInt, &value, NULL, "Word length", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "inchannels");
-    desc->params[i].character = 'i';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.ui = OSS_DRIVER_DEF_INS;
-    strcpy(desc->params[i].short_desc, "Capture channels");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
-    
-    i++;
-    strcpy(desc->params[i].name, "outchannels");
-    desc->params[i].character = 'o';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.ui = OSS_DRIVER_DEF_OUTS;
-    strcpy(desc->params[i].short_desc, "Playback channels");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.ui = OSS_DRIVER_DEF_INS;
+    jack_driver_descriptor_add_parameter(desc, &filler, "inchannels", 'i', JackDriverParamUInt, &value, NULL, "Capture channels", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "excl");
-    desc->params[i].character = 'e';
-    desc->params[i].type = JackDriverParamBool;
-    desc->params[i].value.i = false;
-    strcpy(desc->params[i].short_desc, "Exclusif (O_EXCL) access mode");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.ui = OSS_DRIVER_DEF_OUTS;
+    jack_driver_descriptor_add_parameter(desc, &filler, "outchannels", 'o', JackDriverParamUInt, &value, NULL, "Playback channels", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "capture");
-    desc->params[i].character = 'C';
-    desc->params[i].type = JackDriverParamString;
-    strcpy(desc->params[i].value.str, OSS_DRIVER_DEF_DEV);
-    strcpy(desc->params[i].short_desc, "Input device");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
-    
-    i++;
-    strcpy(desc->params[i].name, "playback");
-    desc->params[i].character = 'P';
-    desc->params[i].type = JackDriverParamString;
-    strcpy(desc->params[i].value.str, OSS_DRIVER_DEF_DEV);
-    strcpy(desc->params[i].short_desc, "Output device");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.i = false;
+    jack_driver_descriptor_add_parameter(desc, &filler, "excl", 'e', JackDriverParamBool, &value, NULL, "Exclusif (O_EXCL) access mode", NULL);
 
-    i++;
-    strcpy (desc->params[i].name, "device");
-    desc->params[i].character = 'd';
-    desc->params[i].type = JackDriverParamString;
-    strcpy(desc->params[i].value.str, OSS_DRIVER_DEF_DEV);
-    strcpy(desc->params[i].short_desc, "OSS device name");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
-  
-    i++;
-    strcpy(desc->params[i].name, "ignorehwbuf");
-    desc->params[i].character = 'b';
-    desc->params[i].type = JackDriverParamBool;
-    desc->params[i].value.i = false;
-    strcpy(desc->params[i].short_desc, "Ignore hardware period size");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
-   
-    i++;
-    strcpy(desc->params[i].name, "input-latency");
-    desc->params[i].character = 'I';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.i = 0;
-    strcpy(desc->params[i].short_desc, "Extra input latency");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    strcpy(value.str, OSS_DRIVER_DEF_DEV);
+    jack_driver_descriptor_add_parameter(desc, &filler, "capture", 'C', JackDriverParamString, &value, NULL, "Input device", NULL);
+    jack_driver_descriptor_add_parameter(desc, &filler, "playback", 'P', JackDriverParamString, &value, NULL, "Output device", NULL);
+    jack_driver_descriptor_add_parameter(desc, &filler, "device", 'd', JackDriverParamString, &value, NULL, "OSS device name", NULL);
 
-    i++;
-    strcpy(desc->params[i].name, "output-latency");
-    desc->params[i].character = 'O';
-    desc->params[i].type = JackDriverParamUInt;
-    desc->params[i].value.i = 0;
-    strcpy(desc->params[i].short_desc, "Extra output latency");
-    strcpy(desc->params[i].long_desc, desc->params[i].short_desc);
+    value.i = false;
+    jack_driver_descriptor_add_parameter(desc, &filler, "ignorehwbuf", 'b', JackDriverParamBool, &value, NULL, "Ignore hardware period size", NULL);
+
+    value.ui = 0;
+    jack_driver_descriptor_add_parameter(desc, &filler, "input-latency", 'I', JackDriverParamUInt, &value, NULL, "Extra input latency", NULL);
+    jack_driver_descriptor_add_parameter(desc, &filler, "output-latency", 'O', JackDriverParamUInt, &value, NULL, "Extra output latency", NULL);
 
     return desc;
 }
@@ -870,13 +795,13 @@ EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine
     bool ignorehwbuf = false;
     jack_nframes_t systemic_input_latency = 0;
     jack_nframes_t systemic_output_latency = 0;
- 
+
     for (node = params; node; node = jack_slist_next(node)) {
-    
+
         param = (const jack_driver_param_t *)node->data;
 
         switch (param->character) {
-        
+
         case 'r':
             srate = param->value.ui;
             break;
@@ -900,7 +825,7 @@ EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine
         case 'o':
             chan_out = (int)param->value.ui;
             break;
-            
+
         case 'C':
             capture = true;
             if (strcmp(param->value.str, "none") != 0) {
@@ -919,7 +844,7 @@ EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine
             playback_pcm_name = param->value.str;
             capture_pcm_name = param->value.str;
             break;
-    
+
         case 'b':
             ignorehwbuf = true;
             break;
@@ -927,7 +852,7 @@ EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine
         case 'e':
             excl = true;
             break;
-		
+
         case 'I':
             systemic_input_latency = param->value.ui;
             break;
@@ -946,9 +871,9 @@ EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine
 
     Jack::JackOSSDriver* oss_driver = new Jack::JackOSSDriver("system", "oss", engine, table);
     Jack::JackDriverClientInterface* threaded_driver = new Jack::JackThreadedDriver(oss_driver);
-    
+
     // Special open for OSS driver...
-    if (oss_driver->Open(frames_per_interrupt, nperiods, srate, capture, playback, chan_in, chan_out, 
+    if (oss_driver->Open(frames_per_interrupt, nperiods, srate, capture, playback, chan_in, chan_out,
         excl, monitor, capture_pcm_name, playback_pcm_name, systemic_input_latency, systemic_output_latency, bits, ignorehwbuf) == 0) {
         return threaded_driver;
     } else {
